class ConfigurationApiData(object):
    ConfigDetail = {
        "response": {
            "private_ipaddress": "10.0.1.211",
            "public_ipaddress": "52.202.254.43",
            "topology_checksum": "9e29c480e3a20f1a9e4fd7f453f8da7280bdada4",
            "vns3_version": "4.8.4-20191220",
            "topology_name": "My-topology-name",
            "ntp_hosts": "0.ubuntu.pool.ntp.org 1.ubuntu.pool.ntp.org 2.ubuntu.pool.ntp.org 3.ubuntu.pool.ntp.org ntp.ubuntu.com time.apple.com",
            "licensed": False,
        }
    }

    KeysetDetail = {
        "response": {
            "keyset_present": True,
            "created_at": "2020-01-24T20:42:56.500+00:00",
            "created_at_i": 1579898576,
            "checksum": "8e4f8ae182a41a047c206c099c62fa0ea59ebbcb",
            "uuid": "184f06f0-3eea-11ea-ad17-02e11c8e6b1d",
        }
    }

    UpdateUIAdminResponse = {"response": {"enabled": True, "username": "vnscubed",}}


class IpsecApiData(object):
    IpsecEndpointDetail = {
        "response": {
            "name": "EndpointB",
            "ipaddress": "13.53.72.182",
            "pfs": True,
            "id": 1,
            "ike_version": 1,
            "nat_t_enabled": True,
            "private_ipaddress": "192.0.2.254",
            "extra_config": [],
            "tunnels": {
                "3": {
                    "id": 3,
                    "local_subnet": "172.31.0.0/28",
                    "remote_subnet": "192.168.10.0/22",
                    "endpoint_id": 1,
                    "enabled": True,
                    "description": "tunnel",
                    "ping_ipaddress": "",
                    "ping_interface": "tun0",
                }
            },
            "bgp_peers": {},
            "type": "ipsec",
            "vpn_type": "policy",
            "psk": "testtest",
        }
    }

    IpsecEndpointDetailNoTunnels = {
        "response": {
            "name": "EndpointC",
            "ipaddress": "13.53.72.182",
            "pfs": True,
            "id": 1,
            "ike_version": 2,
            "nat_t_enabled": True,
            "private_ipaddress": "192.0.2.254",
            "extra_config": [],
            "tunnels": {},
            "bgp_peers": {},
            "type": "ipsec",
            "vpn_type": "policy",
            "psk": "nnotnot",
        }
    }

    IpsecSystemDetailResponse = {
        "response": {
            "remote_endpoints": {
                "1": {
                    "name": "EndpointB",
                    "ipaddress": "13.53.72.182",
                    "pfs": True,
                    "id": 1,
                    "ike_version": 1,
                    "nat_t_enabled": True,
                    "private_ipaddress": "192.0.2.254",
                    "extra_config": [],
                    "tunnels": {
                        "1": {
                            "id": 1,
                            "local_subnet": "172.31.0.0/22",
                            "remote_subnet": "192.168.10.0/28",
                            "endpoint_id": 1,
                            "enabled": True,
                            "description": "Tunnel A",
                            "ping_ipaddress": "",
                            "ping_interface": "tun0",
                            "ping_interval": 30,
                        }
                    },
                    "bgp_peers": {},
                    "type": "ipsec",
                    "vpn_type": "policy",
                    "psk": "testtest",
                }
            },
            "this_endpoint": {
                "ipaddress": "13.48.194.23",
                "overlay_subnet": "172.31.0.0/22",
                "private_ipaddress": "192.0.2.254",
                "ipsec_local_ipaddress": "192.0.2.254",
                "asn": 65001,
            },
        }
    }

    IpsecStatusResponse = {
        "response": {
            "5": {
                "local_subnet": "172.31.0.0/28",
                "remote_subnet": "192.168.10.0/22",
                "endpointid": 5,
                "endpoint_name": "EndpointB",
                "active": True,
                "description": "tunnel",
                "connected": True,
                "tunnel_params": {
                    "phase2": "up",
                    "outbound_spi": "e7a4b70c",
                    "inbound_spi": "64dd6ec8",
                    "bytes_in": "0",
                    "bytes_out": "0",
                    "esp_time_remaining": "20756s",
                    "esp_port": "4500",
                    "phase2_algo": "AES 128",
                    "phase2_hash": "SHA1",
                    "nat_t": "on",
                    "dpd": "off",
                    "pfs_dh_group": 14,
                    "phase1": "up",
                    "isakmp_port": "4500",
                    "isakmp_time_remaining": "1227s",
                    "last_dpd": "None",
                    "phase1_cipher": "AES 256",
                    "phase1_prf": "SHA2_256",
                    "phase1_dh_group": 14,
                    "ike_version": "IKEv1",
                },
            }
        }
    }

    IpsecTunnelDetail = {
        "response": {
            "id": 2,
            "local_subnet": "172.31.0.0/28",
            "remote_subnet": "192.168.10.0/22",
            "endpoint_id": 1,
            "enabled": True,
            "description": "tunnel",
            "ping_ipaddress": "",
            "ping_interface": "",
            "ping_interval": None,
        }
    }

    LinkHistoryResponse = {
        "response": {
            "remote": "172.31.0.0/28",
            "history": [
                {
                    "event": "Tunnel up",
                    "timestamp": "2020-01-24T17:41:51.476Z",
                    "timestamp_i": 1564089901,
                }
            ],
            "local": "192.168.10.0/22",
        }
    }


class BGPApiData(object):
    BGPPeerDetail = {
        "response": {
            "id": 1,
            "ipaddress": "55.55.55.55",
            "asn": 65123,
            "bgp_password": "",
            "access_list": "",
            "add_network_distance": False,
            "add_network_distance_direction": "",
            "add_network_distance_hops": 0,
        }
    }


class AccessApiData(object):
    AccessUrl = {
        "id": 24,
        "url": "https://0.0.0.0:8000/?access=a03875aa97ca232fbfb08225389f135bf212f49e675e171b8e7516f22021469c",
        "created_at": "2020-01-24T17:41:51.476Z",
        "created_ip": "24.222.238.197",
        "description": "Created at 2020-01-24T17:41:51+00:00 for 1h",
        "lifetime": "1h",
        "expires_at": "2020-01-24T18:41:51.484Z",
        "expired": False,
        "last_accessed_at": "2020-01-24T17:41:51.476Z",
        "last_accessed_ip": "2020-01-24T17:41:51.476Z",
    }

    AccessUrlDetail = {"response": AccessUrl}
    AccessUrlListResponse = {"response": [AccessUrl]}
    AccessToken = {
        "id": 1,
        "created_at": "2019-08-19T19:07:49.662Z",
        "token": "12vupgeuate2kbpjiltp",
        "created_ip": "24.222.238.197",
        "description": "sadfasdf",
        "expires_at": "2019-08-19T19:37:49.663Z",
        "lifetime": "30m",
        "refreshes": False,
        "expired": True,
        "last_accessed_at": "2019-08-19T19:07:49.662Z",
        "last_accessed_ip": "2019-08-19T19:07:49.662Z",
    }
    AccessTokenDetail = {"response": AccessToken}
    AccessTokenListResponse = {"response": [AccessToken]}
    AccessUrlDeleteResponse = {"response": "Access url deleted"}
    AccessTokenDeleteResponse = {"response": "Token deleted"}


class FirewallApiData(object):
    pass


class InterfacesApiData(object):
    Interface1 = {
        "id": 4,
        "name": "eth0",
        "interface_type": "system",
        "description": "Auto-created interface",
        "ip_internal": "10.0.1.120",
        "mtu": 9001,
        "enabled": True,
        "status": "Up",
        "mask_bits": "25",
        "gateway": None,
        "system_default": True,
        "ip_external": "3.212.92.239",
        "tags": [],
    }

    Interface2 = {
        "id": 5,
        "name": "lo",
        "interface_type": "system",
        "description": "Auto-created interface",
        "ip_internal": "127.0.0.1",
        "mtu": 65536,
        "enabled": True,
        "status": "Up",
        "mask_bits": "8",
        "gateway": None,
        "system_default": True,
        "ip_external": None,
        "tags": [],
    }

    InterfaceDetail = {"response": Interface2}

    InterfaceListResponse = {"response": [Interface1, Interface2]}

    GREEndpoint = {
        "id": 8,
        "interface_id": 4,
        "name": "gre2",
        "interface_type": "gre",
        "description": "Gre endpoint",
        "ip_internal": "127.0.0.1",
        "mtu": 65536,
        "enabled": True,
        "status": "Up",
        "mask_bits": "8",
        "gateway": None,
        "system_default": False,
        "ip_external": None,
        "tags": [],
        "local_connection_ip": "10.1.0.1",
        "remote_connection_ip": "10.2.0.1",
        "ttl": 255,
    }

    GREEndpointDetail = {"response": GREEndpoint}

    GREEndpointListResponse = {"response": [GREEndpoint]}


class LicensingApiData(object):
    LicenseUploadDetail = {
        "response": {
            "license": "accepted",
            "finalized": False,
            "license_present": True,
            "capabilities": [
                "IPsec",
                "eBGP",
                "LinearAddressing",
                "LinearAddressingConfigurable",
                "CloudWAN",
                "Containers",
            ],
            "default_topology": {
                "managers": [
                    {
                        "manager_id": 1,
                        "overlay_ipaddress": {
                            "ip_address": "100.127.255.253",
                            "octets": [100, 127, 255, 253],
                        },
                        "asn": 65001,
                    },
                    {
                        "manager_id": 2,
                        "overlay_ipaddress": {
                            "ip_address": "100.127.255.252",
                            "octets": [100, 127, 255, 252],
                        },
                        "asn": 65002,
                    },
                    {
                        "manager_id": 3,
                        "overlay_ipaddress": {
                            "ip_address": "100.127.255.251",
                            "octets": [100, 127, 255, 251],
                        },
                        "asn": 65003,
                    },
                    {
                        "manager_id": 4,
                        "overlay_ipaddress": {
                            "ip_address": "100.127.255.250",
                            "octets": [100, 127, 255, 250],
                        },
                        "asn": 65004,
                    },
                ],
                "clients": [
                    {"ip_address": "100.127.255.193", "octets": [100, 127, 255, 193]},
                    {"ip_address": "100.127.255.194", "octets": [100, 127, 255, 194]},
                    {"ip_address": "100.127.255.195", "octets": [100, 127, 255, 195]},
                    {"ip_address": "100.127.255.196", "octets": [100, 127, 255, 196]},
                    {"ip_address": "100.127.255.197", "octets": [100, 127, 255, 197]},
                    {"ip_address": "100.127.255.198", "octets": [100, 127, 255, 198]},
                    {"ip_address": "100.127.255.199", "octets": [100, 127, 255, 199]},
                    {"ip_address": "100.127.255.200", "octets": [100, 127, 255, 200]},
                    {"ip_address": "100.127.255.217", "octets": [100, 127, 255, 217]},
                ],
                "total_clients": 25,
                "overlay_max_clients": 25,
                "overlay_subnet": "100.127.255.192/26",
                "ipsec_max_endpoints": 4,
                "ipsec_max_subnets": 16,
            },
        }
    }

    LicenseDetail = {
        "response": {
            "license_present": True,
            "sha1_checksum": "4320fce04d8db1c5d19a1648990a04e13ffc4419",
            "uploaded_at": "2020-01-24 20:39:59 +0000",
            "uploaded_at_i": 1579898399,
            "topology": {
                "managers": [
                    {
                        "manager_id": 1,
                        "overlay_ipaddress": {
                            "ip_address": "172.31.1.253",
                            "octets": [172, 31, 1, 253],
                        },
                        "asn": 34001,
                    },
                    {
                        "manager_id": 2,
                        "overlay_ipaddress": {
                            "ip_address": "172.31.1.252",
                            "octets": [172, 31, 1, 252],
                        },
                        "asn": 34002,
                    },
                    {
                        "manager_id": 3,
                        "overlay_ipaddress": {
                            "ip_address": "172.31.1.251",
                            "octets": [172, 31, 1, 251],
                        },
                        "asn": 35011,
                    },
                    {
                        "manager_id": 4,
                        "overlay_ipaddress": {
                            "ip_address": "172.31.1.250",
                            "octets": [172, 31, 1, 250],
                        },
                        "asn": 35012,
                    },
                ],
                "clients": [{"ip_address": "172.31.1.1", "octets": [172, 31, 1, 1]}],
                "total_clients": 1,
                "overlay_max_clients": 25,
                "overlay_subnet": "172.31.1.0/24",
                "ipsec_max_endpoints": 4,
                "ipsec_max_subnets": 16,
            },
            "capabilities": [
                "IPsec",
                "eBGP",
                "LinearAddressing",
                "LinearAddressingConfigurable",
                "CloudWAN",
                "Containers",
            ],
            "container_details": {
                "containers_run_count": 4,
                "containers_image_count": 50,
            },
            "finalized": True,
            "custom_addressing": True,
        }
    }

    LicenseParamsResponse = {
        "response": {
            "license": "accepted",
            "finalized": True,
            "parameters": {
                "subnet": "172.31.1.0/24",
                "managers": [
                    "172.31.1.253",
                    "172.31.1.252",
                    "172.31.1.251",
                    "172.31.1.250",
                ],
                "clients": ["172.31.1.1"],
                "asns": [34001, 34002, 35011, 35012],
                "my_manager_vip": "172.31.1.254",
            },
        }
    }

    UpgradeLicenseResponse = {
        "response": {
            "finalized": True,
            "uniq": "4320fce04d8db1c5d19a1648990a04e13ffc4419",
            "license": "in-progress",
            "new_clientpacks": 50,
            "new_managers": 4,
        }
    }


class MonitoringApiData(object):
    Alert = {
        "id": 23,
        "webhook_id": 21,
        "name": "Slack Tunnel Alerts",
        "enabled": True,
        "created_at": "2020-01-14T17:32:55.904Z",
        "updated_at": "2020-01-14T17:32:55.904Z",
        "url": "https://hooks.slack.com/services/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",
        "custom_properties": [],
        "events": ["tunnel_up", "tunnel_down"],
    }

    AlertDetail = {"response": Alert}

    AlertsListResponse = {"response": [Alert]}

    Webhook = {
        "body": "{\n"
        '    "attachments": [{\n'
        '      "author_name": "Tunnel %{status}: '
        '%{tunnel_description}",\n'
        '      "author_link": "%{tunnel_url}",\n'
        '      "fields": [\n'
        '        {"title": "VNS3 Controller", "value": '
        '"%{topo_name}" , "short": True},\n'
        '        {"title": "Endpoint", "value": "%{ipsec_name} '
        '(%{ipsec_ip})", "short": True},\n'
        '        {"title": "Local Subnet", "value": '
        '"%{local_subnet}", "short": True},\n'
        '        {"title": "Remote Subnet", "value": '
        '"%{remote_subnet}", "short": True}\n'
        "      ]\n"
        "    }]\n"
        "  }",
        "created_at": "2019-11-12T16:03:28.554000",
        "custom_properties": [],
        "events": ["tunnel_up", "tunnel_down"],
        "headers": [],
        "id": 2,
        "name": "Test Slack",
        "parameters": [],
        "updated_at": "2019-11-12T16:03:28.554000",
        "url": "",
        "validate_cert": True,
    }

    WebhookDetail = {"response": Webhook}

    WebhookListResponse = {"response": [Webhook]}


class NetworkEdgePluginsApiData(object):
    ContainerLogsResponse = {
        "response": {
            "uuid": "09334531f1892b6a9bbd63c9e8aff64ac237aea18159dd41d645814ba95b733f",
            "logs": [
                "Unlinking stale socket /var/run/supervisor.sock\r",
                "2020-01-24 22:10:57,062 INFO RPC interface 'supervisor' initialized\r",
            ],
        }
    }

    ContainerImage1 = {
        "id": "sha256:f172625acb12cd0299dfc53627e25856bd3d0f51b615b4b3c8cdbdde185243b5",
        "image_name": "HAimage",
        "tag_name": "vns3local:HAimage",
        "status": "Ready",
        "status_msg": None,
        "import_id": "33ecfea886d4243af29f639e20ab6a01f83dec088f563bb2889bbb641bfb5be8",
        "created": "2020-01-24T19:01:20.539Z",
        "description": "",
        "comment": None,
        "container_config": None,
    }

    ContainerImage2 = {
        "id": "sha256:3ba915f8153cb7443aafce77cb4f250b1d0fcb861c297ae7d04ec67f7ec259fa",
        "image_name": "SplunkLogger",
        "tag_name": "vns3local:SplunkLogger",
        "status": "Ready",
        "status_msg": None,
        "import_id": "d809ca4d5cddd7b7639be8611d097377ac94c077267107ee6ff8b88091ca3b4f",
        "created": "2020-01-24T19:02:37.230Z",
        "description": "Splunk logger that configures controller splunk hostname via the command.",
        "comment": None,
        "container_config": None,
    }

    ContainerImagesResponse = {
        "response": {"images": [ContainerImage1, ContainerImage2]}
    }

    ContainerSystemStatus = {
        "response": {"running": True, "network": "198.51.100.0/28"}
    }

    ContainerSystemIPs = {
        "response": {
            "addresses": [
                {"address": "198.51.100.0", "status": "reserved"},
                {"address": "198.51.100.1", "status": "reserved"},
                {"address": "198.51.100.2", "status": "in_use"},
                {"address": "198.51.100.3", "status": "in_use"},
                {"address": "198.51.100.4", "status": "available"},
                {"address": "198.51.100.5", "status": "available"},
                {"address": "198.51.100.6", "status": "available"},
                {"address": "198.51.100.7", "status": "available"},
                {"address": "198.51.100.8", "status": "available"},
            ]
        }
    }

    ContainersListResponse = {
        "response": {
            "containers": [
                {
                    "Id": "09334531f1892b6a9bbd63c9e8aff64ac237aea18159dd41d645814ba95b733f",
                    "Created": "2020-01-24T22:10:56.370194442Z",
                    "Path": "/usr/bin/execStartLogger.sh",
                    "Args": ["ChinaEnv-VNS3-4311-Controller1"],
                    "State": {
                        "Status": "running",
                        "Running": True,
                        "Paused": False,
                        "Restarting": False,
                        "OOMKilled": False,
                        "Dead": False,
                        "Pid": 9507,
                        "ExitCode": 0,
                        "Error": "",
                        "StartedAt": "2020-01-24T22:10:56.581735816Z",
                        "FinishedAt": "0001-01-01T00:00:00Z",
                    },
                    "Image": "sha256:3595cffccf01d1319e8e84507a369d5104340d5e66ab381a2004d60d5b8ac772",
                    "ResolvConfPath": "/var/lib/docker/containers/09334531f1892b6a9bbd63c9e8aff64ac237aea18159dd41d645814ba95b733f/resolv.conf",
                    "HostnamePath": "/var/lib/docker/containers/09334531f1892b6a9bbd63c9e8aff64ac237aea18159dd41d645814ba95b733f/hostname",
                    "HostsPath": "/var/lib/docker/containers/09334531f1892b6a9bbd63c9e8aff64ac237aea18159dd41d645814ba95b733f/hosts",
                    "LogPath": "/var/lib/docker/containers/09334531f1892b6a9bbd63c9e8aff64ac237aea18159dd41d645814ba95b733f/09334531f1892b6a9bbd63c9e8aff64ac237aea18159dd41d645814ba95b733f-json.log",
                    "Name": "/dreamy_haibt",
                    "RestartCount": 0,
                    "Driver": "aufs",
                    "MountLabel": "",
                    "ProcessLabel": "",
                    "AppArmorProfile": "",
                    "ExecIDs": None,
                    "HostConfig": {
                        "Binds": ["/mnt/logs:/mnt/logs:rw"],
                        "ContainerIDFile": "",
                        "LogConfig": {"Type": "json-file", "Config": {}},
                        "NetworkMode": "cohesive.net",
                        "PortBindings": {},
                        "RestartPolicy": {"Name": "always", "MaximumRetryCount": 0},
                        "AutoRemove": False,
                        "VolumeDriver": "",
                        "VolumesFrom": None,
                        "CapAdd": None,
                        "CapDrop": None,
                        "Dns": [],
                        "DnsOptions": [],
                        "DnsSearch": [],
                        "ExtraHosts": None,
                        "GroupAdd": None,
                        "IpcMode": "",
                        "Cgroup": "",
                        "Links": None,
                        "OomScoreAdj": 0,
                        "PidMode": "",
                        "Privileged": True,
                        "PublishAllPorts": False,
                        "ReadonlyRootfs": False,
                        "SecurityOpt": ["label=disable"],
                        "UTSMode": "",
                        "UsernsMode": "",
                        "ShmSize": 67108864,
                        "Runtime": "runc",
                        "ConsoleSize": [0, 0],
                        "Isolation": "",
                        "CpuShares": 0,
                        "Memory": 0,
                        "NanoCpus": 0,
                        "CgroupParent": "",
                        "BlkioWeight": 0,
                        "BlkioWeightDevice": None,
                        "BlkioDeviceReadBps": None,
                        "BlkioDeviceWriteBps": None,
                        "BlkioDeviceReadIOps": None,
                        "BlkioDeviceWriteIOps": None,
                        "CpuPeriod": 0,
                        "CpuQuota": 0,
                        "CpuRealtimePeriod": 0,
                        "CpuRealtimeRuntime": 0,
                        "CpusetCpus": "",
                        "CpusetMems": "",
                        "Devices": [],
                        "DiskQuota": 0,
                        "KernelMemory": 0,
                        "MemoryReservation": 0,
                        "MemorySwap": 0,
                        "MemorySwappiness": -1,
                        "OomKillDisable": False,
                        "PidsLimit": 0,
                        "Ulimits": None,
                        "CpuCount": 0,
                        "CpuPercent": 0,
                        "IOMaximumIOps": 0,
                        "IOMaximumBandwidth": 0,
                    },
                    "GraphDriver": {"Name": "aufs", "Data": None},
                    "Volumes": {"/mnt/logs": "/mnt/logs"},
                    "VolumesRW": {"/mnt/logs": True},
                    "Config": {
                        "Hostname": "09334531f189",
                        "Domainname": "",
                        "User": "",
                        "AttachStdin": False,
                        "AttachStdout": False,
                        "AttachStderr": False,
                        "Tty": True,
                        "OpenStdin": False,
                        "StdinOnce": False,
                        "Env": ["oe_name=SplunkLogger"],
                        "Cmd": [
                            "/usr/bin/execStartLogger.sh",
                            "ChinaEnv-VNS3-4311-Controller1",
                        ],
                        "Image": "sha256:3595cffccf01d1319e8e84507a369d5104340d5e66ab381a2004d60d5b8ac772",
                        "Volumes": None,
                        "WorkingDir": "",
                        "Entrypoint": None,
                        "OnBuild": None,
                        "Labels": {},
                        "MacAddress": "",
                        "NetworkDisabled": False,
                        "ExposedPorts": None,
                        "VolumeDriver": "",
                        "Memory": 0,
                        "MemorySwap": 0,
                        "CpuShares": 0,
                        "Cpuset": "",
                    },
                    "NetworkSettings": {
                        "Bridge": "",
                        "SandboxID": "1b917be5ff5b9e30d2e6d7ea23b12023f0158a162618fa237b9abf0c86bd825d",
                        "HairpinMode": False,
                        "LinkLocalIPv6Address": "",
                        "LinkLocalIPv6PrefixLen": 0,
                        "Ports": {},
                        "SandboxKey": "/var/run/docker/netns/1b917be5ff5b",
                        "SecondaryIPAddresses": None,
                        "SecondaryIPv6Addresses": None,
                        "EndpointID": "",
                        "Gateway": "",
                        "GlobalIPv6Address": "",
                        "GlobalIPv6PrefixLen": 0,
                        "IPAddress": "",
                        "IPPrefixLen": 0,
                        "IPv6Gateway": "",
                        "MacAddress": "",
                    },
                },
                {
                    "Id": "5c22b4e31cccfc568b770fd576b41865f36aee1293cd354cbee5494cc1e81f21",
                    "Created": "2020-01-24T22:10:50.979930165Z",
                    "Path": "/usr/bin/supervisord",
                    "Args": [],
                    "State": {
                        "Status": "running",
                        "Running": True,
                        "Paused": False,
                        "Restarting": False,
                        "OOMKilled": False,
                        "Dead": False,
                        "Pid": 9221,
                        "ExitCode": 0,
                        "Error": "",
                        "StartedAt": "2020-01-24T22:10:51.299339771Z",
                        "FinishedAt": "0001-01-01T00:00:00Z",
                    },
                    "Image": "sha256:dd354ae9d0be5003dbcee9fe777446aa1b09e159af324a6c22be0c5d1f37250f",
                    "ResolvConfPath": "/var/lib/docker/containers/5c22b4e31cccfc568b770fd576b41865f36aee1293cd354cbee5494cc1e81f21/resolv.conf",
                    "HostnamePath": "/var/lib/docker/containers/5c22b4e31cccfc568b770fd576b41865f36aee1293cd354cbee5494cc1e81f21/hostname",
                    "HostsPath": "/var/lib/docker/containers/5c22b4e31cccfc568b770fd576b41865f36aee1293cd354cbee5494cc1e81f21/hosts",
                    "LogPath": "/var/lib/docker/containers/5c22b4e31cccfc568b770fd576b41865f36aee1293cd354cbee5494cc1e81f21/5c22b4e31cccfc568b770fd576b41865f36aee1293cd354cbee5494cc1e81f21-json.log",
                    "Name": "/sleepy_stonebraker",
                    # "RestartCount": 0,
                    "Driver": "aufs",
                    "MountLabel": "",
                    "ProcessLabel": "",
                    "AppArmorProfile": "",
                    "ExecIDs": None,
                    "HostConfig": {
                        "Binds": None,
                        "ContainerIDFile": "",
                        "LogConfig": {"Type": "json-file", "Config": {}},
                        "NetworkMode": "cohesive.net",
                        "PortBindings": {},
                        "RestartPolicy": {"Name": "always", "MaximumRetryCount": 0},
                        "AutoRemove": False,
                        "VolumeDriver": "",
                        "VolumesFrom": None,
                        "CapAdd": None,
                        "CapDrop": None,
                        "Dns": [],
                        "DnsOptions": [],
                        "DnsSearch": [],
                        "ExtraHosts": None,
                        "GroupAdd": None,
                        "IpcMode": "",
                        "Cgroup": "",
                        "Links": None,
                        "OomScoreAdj": 0,
                        "PidMode": "",
                        "Privileged": False,
                        "PublishAllPorts": False,
                        "ReadonlyRootfs": False,
                        "SecurityOpt": None,
                        "UTSMode": "",
                        "UsernsMode": "",
                        "ShmSize": 67108864,
                        "Runtime": "runc",
                        "ConsoleSize": [0, 0],
                        "Isolation": "",
                        "CpuShares": 0,
                        "Memory": 0,
                        "NanoCpus": 0,
                        "CgroupParent": "",
                        "BlkioWeight": 0,
                        "BlkioWeightDevice": None,
                        "BlkioDeviceReadBps": None,
                        "BlkioDeviceWriteBps": None,
                        "BlkioDeviceReadIOps": None,
                        "BlkioDeviceWriteIOps": None,
                        "CpuPeriod": 0,
                        "CpuQuota": 0,
                        "CpuRealtimePeriod": 0,
                        "CpuRealtimeRuntime": 0,
                        "CpusetCpus": "",
                        "CpusetMems": "",
                        "Devices": [],
                        "DiskQuota": 0,
                        "KernelMemory": 0,
                        "MemoryReservation": 0,
                        "MemorySwap": 0,
                        "MemorySwappiness": -1,
                        "OomKillDisable": False,
                        "PidsLimit": 0,
                        "Ulimits": None,
                        "CpuCount": 0,
                        "CpuPercent": 0,
                        "IOMaximumIOps": 0,
                        "IOMaximumBandwidth": 0,
                    },
                    "GraphDriver": {"Name": "aufs", "Data": None},
                    "Volumes": {},
                    "VolumesRW": {},
                    "Config": {
                        "Hostname": "5c22b4e31ccc",
                        "Domainname": "",
                        "User": "",
                        "AttachStdin": False,
                        "AttachStdout": False,
                        "AttachStderr": False,
                        "Tty": True,
                        "OpenStdin": False,
                        "StdinOnce": False,
                        "Env": None,
                        "Cmd": ["/usr/bin/supervisord"],
                        "Image": "sha256:dd354ae9d0be5003dbcee9fe777446aa1b09e159af324a6c22be0c5d1f37250f",
                        "Volumes": None,
                        "WorkingDir": "",
                        "Entrypoint": None,
                        "OnBuild": None,
                        "Labels": {},
                        "MacAddress": "",
                        "NetworkDisabled": False,
                        "ExposedPorts": None,
                        "VolumeDriver": "",
                        "Memory": 0,
                        "MemorySwap": 0,
                        "CpuShares": 0,
                        "Cpuset": "",
                    },
                    "NetworkSettings": {
                        "Bridge": "",
                        "SandboxID": "7c310373310a0b1fb2336389c90d332b43334a77dc9d1caaca11535c80da890b",
                        "HairpinMode": False,
                        "LinkLocalIPv6Address": "",
                        "LinkLocalIPv6PrefixLen": 0,
                        "Ports": {},
                        "SandboxKey": "/var/run/docker/netns/7c310373310a",
                        "SecondaryIPAddresses": None,
                        "SecondaryIPv6Addresses": None,
                        "EndpointID": "",
                        "Gateway": "",
                        "GlobalIPv6Address": "",
                        "GlobalIPv6PrefixLen": 0,
                        "IPAddress": "",
                        "IPPrefixLen": 0,
                        "IPv6Gateway": "",
                        "MacAddress": "",
                    },
                },
            ]
        }
    }


class OverlayNetworkApiData(object):
    Clientpack = {
        "overlay_ipaddress": "192.168.10.1",
        "name": "192_168_10_1",
        "tarball_file": "192_168_10_1.tar.gz",
        "tarball_sha1": "61a24dbcf764da153fedfdb4c8c02375c25ab4f9",
        "sequential_id": 1,
        "zip_file": "192_168_10_1.zip",
        "zip_sha1": "2d7ef3717d417b59091012ee7f7168bf96cf309e",
        "linux_onefile": "192_168_10_1.conf",
        "conf_sha1": "d504c924cb9cb7a24d01701475ff68ca7d76ca0c",
        "windows_onefile": "192_168_10_1.ovpn",
        "ovpn_sha1": "d504c924cb9cb7a24d01701475ff68ca7d76ca0c",
        "status": "not_connected",
        "enabled": True,
        "checked_out": False,
        "tags": {},
        "last_connect": "",
        "last_disconnect": "",
    }

    NextClientpackResponse = {
        "response": {
            "name": "172_31_0_2",
            "overlay_ipaddress": "172.31.0.2",
            "enabled": True,
            "checked_out": True,
            "sequential_id": 2,
            "linux_onefile": "172_31_0_2.conf",
            "conf_sha1": "e31cc4db37ce2b082d0b08dc73b6af27ac0ff9ae",
            "windows_onefile": "172_31_0_2.ovpn",
            "ovpn_sha1": "e31cc4db37ce2b082d0b08dc73b6af27ac0ff9ae",
            "tarball_file": "172_31_0_2.tar.gz",
            "tarball_sha1": "b9a2e4961d2519154af488ea5572fe083e624ea7",
            "zip_file": "172_31_0_2.zip",
            "zip_sha1": "21340d31bf939da497f042a5eaecb6a9d7dbd947",
        }
    }

    ClientpackDetail = {"response": {"clientpack": Clientpack}}

    ClientpacksListResponse = {"response": {"192.168.10.1": Clientpack}}

    ClientpackTagsResponse = {
        "response": {
            "name": "my-name",
            "tags": {"environment": "chicago", "application": "webapp"},
        }
    }


class PeeringApiData(object):
    PeeringStatus = {
        "response": {
            "peered": True,
            "managers": {
                "1": {"overlay_ipaddress": "172.31.1.253", "self": True, "id": 1},
                "2": {"overlay_ipaddress": "172.31.1.252", "not_set": True, "id": 2},
                "3": {"overlay_ipaddress": "172.31.1.251", "not_set": True, "id": 3},
                "4": {"overlay_ipaddress": "172.31.1.250", "not_set": True, "id": 4},
            },
            "id": 1,
        }
    }


class RoutingApiData(object):
    RoutesResponse = {
        "response": {
            "1": {
                "cidr": "224.0.0.0/4",
                "id": 1,
                "netmask": "240.0.0.0",
                "interface": "tun0",
                "description": "Multicast (auto-added)",
                "advertise": False,
                "metric": 0,
            },
            "2": {
                "cidr": "10.121.11.212/32",
                "id": 3,
                "netmask": "255.255.255.255",
                "interface": "eth0",
                "gateway": "10.121.11.209",
                "description": "Local Route subnet gateway",
                "advertise": False,
                "metric": 0,
            },
            "3": {
                "cidr": "10.121.11.213/32",
                "id": 4,
                "netmask": "255.255.255.255",
                "interface": "eth0",
                "gateway": "10.121.11.209",
                "description": "Local Secondary IP",
                "advertise": False,
                "metric": 0,
            },
            "4": {
                "cidr": "10.0.0.0/8",
                "id": 8,
                "netmask": "255.0.0.0",
                "interface": "_notset",
                "description": "On Prem AWS",
                "advertise": True,
                "metric": 0,
            },
        }
    }


class SnapshotsApiData(object):
    SnapshotData = {
        "snapshot_20200127_1580154368_3.212.92.239": {
            "sha1_checksum": "869c0dc6bfa3f811b1f7affee4051a29a7ecac8b",
            "created_at": "2020-01-27T19:46:09.634+00:00",
            "created_at_i": 1580154369,
            "size": 1479421,
        }
    }

    SnapshotDetail = {"response": SnapshotData}

    SnapshotsListResponse = {
        "response": {
            "latest_snapshot": "snapshot_20200127_1580154368_3.212.92.239",
            "snapshots": SnapshotData,
        }
    }

    SnapshotImportResponse = {"response": {"snapshot": "accepted"}}


class SystemAdminApiData(object):
    CloudDataDetail = {
        "response": {
            "cloud_data": {
                "accountId": "947761958079",
                "architecture": "x86_64",
                "availabilityZone": "us-east-1a",
                "billingProducts": None,
                "devpayProductCodes": None,
                "imageId": "ami-034b377bde620d2a8",
                "instanceId": "i-0b96e779c3143cf66",
                "instanceType": "t3.small",
                "kernelId": None,
                "marketplaceProductCodes": None,
                "pendingTime": "2019-11-08T16:18:38Z",
                "privateIp": "10.0.1.37",
                "ramdiskId": None,
                "region": "us-east-1",
                "version": "2017-09-30",
            },
            "cloud_type": "ec2",
        }
    }

    RemoteSupportDetails = {"response": {"enabled": True}}

    UpdateRemoteSupportResponse = {
        "response": {"enabled": False, "revoke_credential": True}
    }

    VNS3Status = {
        "response": {
            "connected_clients": {},
            "connected_subnets": [["10.0.1.0", "255.255.255.0"]],
            "ipsec": {},
        }
    }

    SystemStatus = {
        "response": {
            "data": {
                "sysstat": [
                    ["vnscubed", "600", "1580160301", "all", "%user", "2.16"],
                    ["vnscubed", "600", "1580160301", "all", "%nice", "0.01"],
                    ["vnscubed", "600", "1580160301", "cpu0", "%user", "2.23"],
                    ["vnscubed", "600", "1580160301", "cpu0", "%nice", "0.01"],
                    ["vnscubed", "600", "1580160301", "cpu0", "%system", "1.26"],
                    ["vnscubed", "600", "1580160301", "cpu0", "%iowait", "0.02"],
                ]
            },
            "timestamp": "2020-01-27 21:46:35 +0000",
            "timestamp_i": 1580161595,
            "vns3_version": "Cohesive Networks VNS3 4.9.0-20200121",
            "kernel_version": "Linux 4.4.0-154-generic",
            "uptime": 263155,
            "loadavg": ["0.00", "0.05", "0.07", "1/328", "20398"],
            "diskinfo": [
                [
                    "/dev/nvme0n1p1",
                    "31571570688",
                    "5974278144",
                    "23986741248",
                    "20%",
                    "/",
                    "13%",
                ]
            ],
            "meminfo": [
                "8264527872",
                "4446773248",
                "3817754624",
                "90910720",
                "241074176",
                "3327860736",
                "877838336",
                "7386689536",
            ],
            "swapinfo": ["1073737728", "0", "1073737728"],
            "container_system": {
                "container_system_running": True,
                "images_limit": 50,
                "images_stored": 3,
                "containers_limit": 4,
                "containers_active": 1,
                "container_network": "198.51.100.0/28",
            },
        }
    }



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from cohesivenet.api.vns3ms.access_api import AccessApiRouter as AccessApi
from cohesivenet.api.vns3ms.administration_api import (
    AdministrationApiRouter as AdministrationApi,
)
from cohesivenet.api.vns3ms.backups_api import BackupsApiRouter as BackupsApi
from cohesivenet.api.vns3ms.cloud_monitoring_api import (
    CloudMonitoringApiRouter as CloudMonitoringApi,
)
from cohesivenet.api.vns3ms.system_api import SystemApiRouter as SystemApi
from cohesivenet.api.vns3ms.user_api import UserApiRouter as UserApi
from cohesivenet.api.vns3ms.vns3_management_api import (
    VNS3ManagementApiRouter as VNS3ManagementApi,
)



# coding: utf-8

"""
    VNS3:ms API

    Cohesive networks VNS3 provides complete control of your network's addresses, routes, rules and edge. Networking does  # noqa: E501

    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from cohesivenet.api_builder import VersionRouter


def put_enable_ldap(api_client, enabled=None, **kwargs):  # noqa: E501
    """Enable/Disable LDAP  # noqa: E501

    Enable/Disable LDAP  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_enable_ldap(async_req=True)

    :param VNS3Client api_client: (required)
    :param enabled bool: Enable/Disable LDAP
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["enabled"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_ldap_settings(api_client, **kwargs):  # noqa: E501
    """Get LDAP Settings  # noqa: E501

    Get LDAP Settings  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_ldap_settings(async_req=True)

    :param VNS3Client api_client: (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/settings",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_ldap_settings(
    api_client,
    ldap_host=None,
    ldap_port=None,
    ldap_ssl=None,
    ldap_encrypt=None,
    ldap_encrypt_ldaps=None,
    ldap_encrypt_auth=None,
    ldap_encrypt_verify_ca=None,
    ldap_binddn=None,
    ldap_bindpw=None,
    **kwargs
):  # noqa: E501
    """Update LDAP connection configuration settings  # noqa: E501

    Update LDAP connection configuration settings  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_ldap_settings(async_req=True)

    :param VNS3Client api_client: (required)
    :param ldap_host str: Host for LDAP server, IP or DNS (required)
    :param ldap_port int: Port for LDAP server
    :param ldap_ssl bool: Use SSL?
    :param ldap_encrypt bool: SSL enabled
    :param ldap_encrypt_ldaps bool: Use LDAPS or start TLS (default)
    :param ldap_encrypt_auth bool: Use certificates to authenticate via encrypted connection
    :param ldap_encrypt_verify_ca bool: Verify certicate using authority
    :param ldap_binddn str: Bind username (required)
    :param ldap_bindpw str: Bind password (required)

    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = [
        "ldap_host",
        "ldap_port",
        "ldap_ssl",
        "ldap_encrypt",
        "ldap_encrypt_ldaps",
        "ldap_encrypt_auth",
        "ldap_encrypt_verify_ca",
        "ldap_binddn",
        "ldap_bindpw",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/settings",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_validate_ldap_settings(
    api_client,
    ldap_host=None,
    ldap_port=None,
    ldap_ssl=None,
    ldap_binddn=None,
    ldap_bindpw=None,
    **kwargs
):  # noqa: E501
    """Validate LDAP connection settings  # noqa: E501

    Validate LDAP connection settings  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_test_ldap_settings(async_req=True)

    :param VNS3Client api_client: (required)
    :param ldap_host str: Host for LDAP server, IP or DNS
    :param ldap_port int: Port for LDAP server
    :param ldap_ssl bool: Use SSL?
    :param ldap_binddn str: Bind username (required)
    :param ldap_bindpw str: Bind password (required)

    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = [
        "ldap_host",
        "ldap_port",
        "ldap_ssl",
        "ldap_binddn",
        "ldap_bindpw",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/settings",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_ldap_user_schema(api_client, **kwargs):  # noqa: E501
    """Get LDAP user schema Settings  # noqa: E501

    Get LDAP user schema Settings  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_ldap_user_schema(async_req=True)

    :param VNS3Client api_client: (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/user_schema",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_ldap_user_schema(
    api_client,
    ldap_user_base=None,
    ldap_user_id_attribute=None,
    ldap_user_list_filter=None,
    **kwargs
):  # noqa: E501
    """Set LDAP user schema settings  # noqa: E501

    Set LDAP user schema settings  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_ldap_user_schema(async_req=True)

    :param VNS3Client api_client: (required)
    :param ldap_user_base str: Base DN from which to search for Users (required)
    :param ldap_user_id_attribute str: Attribute type for the Users (required)
    :param ldap_user_list_filter str: Search filter for Users

    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = [
        "ldap_user_base",
        "ldap_user_id_attribute",
        "ldap_user_list_filter",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/user_schema",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_validate_ldap_user_schema(
    api_client,
    ldap_user_base=None,
    ldap_user_id_attribute=None,
    ldap_user_list_filter=None,
    limit=None,
    **kwargs
):  # noqa: E501
    """Validate LDAP user schema  # noqa: E501

    Validate LDAP user schema  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_test_ldap_settings(async_req=True)

    :param VNS3Client api_client: (required)
    :param ldap_user_base str: Base DN from which to search for Users (required)
    :param ldap_user_id_attribute str: Attribute type for the Users (required)
    :param ldap_user_list_filter str: Search filter for Users
    :param limit int: Number of records to return. Default = 100

    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = [
        "ldap_user_base",
        "ldap_user_id_attribute",
        "ldap_user_list_filter",
        "limit",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/user_schema",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_ldap_group_schema(api_client, **kwargs):  # noqa: E501
    """Get LDAP group schema Settings  # noqa: E501

    Get LDAP group schema Settings  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_ldap_group_schema(async_req=True)

    :param VNS3Client api_client: (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/group_schema",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_ldap_group_schema(
    api_client,
    ldap_group_required=None,
    ldap_group_base=None,
    ldap_group_id_attribute=None,
    ldap_group_list_filter=None,
    ldap_group_member_attribute=None,
    ldap_group_member_attr_format=None,
    ldap_search_scope=None,
    **kwargs
):  # noqa: E501
    """Set LDAP group schema settings  # noqa: E501

    Set LDAP group schema settings  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_ldap_group_schema(async_req=True)

    :param VNS3Client api_client: (required)
    :param ldap_group_required bool: Require use of LDAP groups (required)
    :param ldap_group_base str: Base DN from which to search for Groups (required)
    :param ldap_group_id_attribute str: Attribute type for the Groups (required)
    :param ldap_group_list_filter str: Search filter for Groups
    :param ldap_group_member_attribute str: Attribute used to search for a user within the Group (required)
    :param ldap_group_member_attr_format str: Format of the Group Member attribute
    :param ldap_search_scope str: Search filter for Groups

    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = [
        "ldap_group_required",
        "ldap_group_base",
        "ldap_group_id_attribute",
        "ldap_group_list_filter",
        "ldap_group_member_attribute",
        "ldap_group_member_attr_format",
        "ldap_search_scope",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/group_schema",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_validate_ldap_group_schema(
    api_client,
    ldap_group_base=None,
    ldap_group_id_attribute=None,
    ldap_group_list_filter=None,
    ldap_group_member_attribute=None,
    ldap_group_member_attr_format=None,
    ldap_search_scope=None,
    limit=None,
    **kwargs
):  # noqa: E501
    """Validate LDAP Group Schema  # noqa: E501

    Validate LDAP Group Schema  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_validate_ldap_group_schema(async_req=True)

    :param VNS3Client api_client: (required)
    :param ldap_group_base str: Base DN from which to search for Groups (required)
    :param ldap_group_id_attribute str: Attribute type for the Groups (required)
    :param ldap_group_list_filter str: Search filter for Groups
    :param ldap_group_member_attribute str: Attribute used to search for a user within the Group (required)
    :param ldap_group_member_attr_format str: Format of the Group Member attribute
    :param ldap_search_scope str: Search filter for Groups
    :param limit int: Number of records to return. Default = 100

    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = [
        "ldap_group_base",
        "ldap_group_id_attribute",
        "ldap_group_list_filter",
        "ldap_group_member_attribute",
        "ldap_group_member_attr_format",
        "ldap_search_scope",
        "limit",
    ]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/group_schema",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_upload_ldap_auth_cert(api_client, body=None, **kwargs):  # noqa: E501
    """put_upload_ldap_auth_cert  # noqa: E501

    Upload LDAP authentication certicate file  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_upload_ldap_auth_cert(body, async_req=True)

    :param async_req bool: execute request asynchronously
    :param body str: cert file (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """
    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []

    local_var_files = {}

    body_params = body

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["text/plain"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/encrypt_auth_cert",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_upload_ldap_auth_key(api_client, body=None, **kwargs):  # noqa: E501
    """put_upload_ldap_auth_key  # noqa: E501

    Upload LDAP authentication key file  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_upload_ldap_auth_key(body, async_req=True)

    :param async_req bool: execute request asynchronously
    :param body str: auth key file (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """
    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []

    local_var_files = {}

    body_params = body

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["text/plain"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/encrypt_auth_key",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_upload_ldap_ca_cert(api_client, body=None, **kwargs):  # noqa: E501
    """put_upload_ldap_ca_cert  # noqa: E501

    Upload LDAP CA certicate file  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_upload_ldap_ca_cert(body, async_req=True)

    :param async_req bool: execute request asynchronously
    :param body str: ca cert file (required)
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """
    local_var_params = locals()

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []

    local_var_files = {}

    body_params = body

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["text/plain"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/admin/ldap/encrypt_ca_cert",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


class AdministrationApiRouter(VersionRouter):
    """Configure VNS3:ms administration settings such as LDAP integration"""

    function_library = {
        "put_enable_ldap": {"2.1.1-2.5.1": put_enable_ldap},
        "get_ldap_settings": {"2.1.1-2.5.1": get_ldap_settings},
        "put_ldap_settings": {"2.1.1-2.5.1": put_ldap_settings},
        "post_validate_ldap_settings": {"2.1.1-2.5.1": post_validate_ldap_settings},
        "get_ldap_user_schema": {"2.1.1-2.5.1": get_ldap_user_schema},
        "put_ldap_user_schema": {"2.1.1-2.5.1": put_ldap_user_schema},
        "post_validate_ldap_user_schema": {
            "2.1.1-2.5.1": post_validate_ldap_user_schema
        },
        "get_ldap_group_schema": {"2.1.1-2.5.1": get_ldap_group_schema},
        "put_ldap_group_schema": {"2.1.1-2.5.1": put_ldap_group_schema},
        "post_validate_ldap_group_schema": {
            "2.1.1-2.5.1": post_validate_ldap_group_schema
        },
        "put_upload_ldap_auth_cert": {
            "2.5.1": put_upload_ldap_auth_cert
        },
        "put_upload_ldap_auth_key": {
            "2.5.1": put_upload_ldap_auth_key
        },
        "put_upload_ldap_ca_cert": {
            "2.5.1": put_upload_ldap_ca_cert
        },
    }



import json
import math
import os
import time
import re

from contextlib import contextmanager
from typing import Dict, List


def take(iterator, count):
    return [next(iterator) for _ in range(count)]


def take_keys(keys: List[str], data_dict: Dict):
    """Take keys from dict

    Arguments:
        keys {List[str]} -- Keys it include in output dict
        data_dict {Dict}

    Returns:
        [Dict]
    """
    return {k: v for k, v in data_dict.items() if k in keys}


def flatten_dict(d, prefix=None, joinchar="__"):
    """flatten_dict Flatten nested dictionary, joining paths into single string key

    Arguments:
        d {[type]} -- [description]

    Keyword Arguments:
        prefix {str} -- Prefix string for joining keys (default: {None})
        joinchar {str} -- String for joining nested keys (default: {'__'})

    Returns:
        [Dict] -- Dict of depth 1
    """
    key_value_pairs = {}

    def _prefix(k):
        return k if not prefix else "%s%s%s" % (prefix, joinchar, k)

    for k, v in d.items():
        if type(v) is dict:
            key_value_pairs.update(
                flatten_dict(v, prefix=_prefix(k), joinchar=joinchar)
            )
        else:
            key_value_pairs[_prefix(k)] = v
    return key_value_pairs


def unflatten_dict(d, splitchar="__"):
    """unflatten_dict Build nested dictionary based on keys

    Arguments:
        d {Dict}

    Keyword Arguments:
        splitchar {str} -- str marking nested dictionary (default: {'__'})

    Returns:
        [Dict]
    """
    resp = {}
    for flatkey, v in d.items():
        keyparts = flatkey.split(splitchar)
        _target = resp
        final_key = keyparts[-1]
        for key in keyparts[:-1]:
            if key not in _target:
                _target[key] = {}
            _target = _target[key]
        _target[final_key] = v
    return resp


def map_type(s, expected_type, strict=True):
    _def_return_val = s
    try:
        if type(s) is list:
            return s
        elif "list" in str(expected_type).lower():
            _def_return_val = []
            return json.loads(s)
        elif not s or s.lower() in ("none", ""):
            return None
        elif expected_type is bool:
            return s.lower() == "true"

        return expected_type(s)
    except ValueError:
        if not strict:
            return _def_return_val
        raise


def is_formattable_string(s):
    if type(s) not in (str, bytes):
        return False

    matches = re.findall(r"{[A-Za-z0-9_\.\[\]]*}", s)
    if matches:
        return [m.strip("{}") for m in matches]
    return False


def format_string(s, state):
    err_none = None
    if is_formattable_string(s):
        try:
            return s.format(**state), err_none
        except KeyError as e:
            return (s, "String format error: missing state args %s" % ",".join(e.args))
    return s, err_none


def dumb_replace(s, value):
    if type(s) not in (str, bytes):
        return s
    return re.sub(r"{.*}", value, s)


def is_list_index(s):
    match = re.match(r"(.*)\[(\d+)\]", str(s))
    if match:
        return match.group(1), match.group(2)
    return False


def get_path(data_dict, key_path, fail=False):
    """get_path

    Arguments:
        data_dict {dict} -- [description]
        key_path {str or List of strings} -- a.b.c or ['a', 'b', 'c']

    Keyword Arguments:
        fail {bool} -- Raise exception if does not exist (default: {False})

    Raises:
        Exception: Generic not found exception

    Returns:
        [any] -- value at key path
    """
    DOES_NOT_EXIST = 'Path "%s" does not exist' % key_path
    _target = data_dict
    steps = key_path if type(key_path) is list else key_path.split(".")

    for step in steps:
        if type(_target) is not dict:
            if fail:
                raise Exception(DOES_NOT_EXIST)
            return None

        step_list_index = is_list_index(step)
        list_index = None
        if step_list_index:
            step, list_index = step_list_index

        if step not in _target:
            if fail:
                raise Exception(DOES_NOT_EXIST)
            return None

        _target = _target.get(step)
        if list_index is not None:
            if type(_target) not in (list, tuple) or len(_target) <= int(list_index):
                if fail:
                    raise Exception(DOES_NOT_EXIST)
                return None
            _target = _target[int(list_index)]

    return _target


def map_dict_values(func_map, data_dict):
    return {k: func_map.get(k, lambda x: x)(v) for k, v in data_dict.items()}


def map_dict_keypaths(key_map, data_dict):
    updates = {
        new_key: get_path(data_dict, key_path) for key_path, new_key in key_map.items()
    }
    return {**data_dict, **updates}


def partition_list_groups(object_list, number_partitions):
    """Partition list of objects into groups

    Arguments:
        object_list {List[Any]}
        number_partitions {int} -- [description]

    Returns:
        [List[List[Any]]]
    """
    if number_partitions <= 1:
        return object_list

    object_count = len(object_list)
    partition_size = math.floor(object_count / number_partitions)
    leftovers = object_count % number_partitions
    leftover_set = object_list[-leftovers:] if leftovers else []
    rounded_list = object_list[:-leftovers] if leftovers else object_list

    return [
        rounded_list[i * partition_size : (i + 1) * partition_size]
        + ([leftover_set[i]] if len(leftover_set) > i else [])
        for i in range(number_partitions)
    ]


def partition_list_ratios(object_list, partition_ratios):
    """Partition list of objects into groups based on ratios list

    Arguments:
        object_list {List[Any]}
        partition_ratios {List[float]}

    Returns:
        Dict[str, List[Any]] - {
            '0.45': [...],
            '0.25': [...],
            '0.30': [...]
        }
    """
    assert math.isclose(sum(partition_ratios), 1.0), "Ratios must sum to 1"

    number_partitions = len(partition_ratios)
    if number_partitions <= 1:
        return object_list

    partition_sizes = [round(r * len(object_list)) for r in partition_ratios]
    _cursor = 0
    partitions = {}
    for i, size in enumerate(partition_sizes):
        partitions[str(partition_ratios[i])] = object_list[_cursor : (_cursor + size)]
        _cursor += size
    return partitions


def random_timestamp_filename(file_type=None):
    timestr = str(time.time()).replace(".", "_")
    if file_type:
        return "%s.%s" % (timestr, file_type)
    return timestr


@contextmanager
def cd(newdir):
    prevdir = os.getcwd()
    os.chdir(os.path.expanduser(newdir))
    try:
        yield
    finally:
        os.chdir(prevdir)


def version_dot_to_int(version_str):
    return int(str(version_str).replace(".", ""))


def version_in_range(version_str, version_range):
    version = version_dot_to_int(version_str)

    if type(version_range) is str:
        range_parts = version_range.split("-")
    else:
        assert type(version_range) is list
        range_parts = version_range

    range_parts_ints = [version_dot_to_int(p) for p in range_parts]
    assert (
        len(range_parts_ints) == 2
    ), "Range invalid. Must be string X.X.X-Y.Y.Y or list"

    return range_parts_ints[0] <= version <= range_parts_ints[1]



# coding: utf-8

"""
    VNS3 Controller API

    Cohesive networks VNS3 API providing complete control of your network's addresses, routes, rules and edge  # noqa: E501

    The version of the OpenAPI document: 4.8
    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import pytest

import cohesivenet
from cohesivenet.api.vns3 import interfaces_api  # noqa: E501
from cohesivenet.rest import ApiException

from tests.openapi import generate_method_test
from tests.vns3.stub_data import InterfacesApiData


class TestInterfacesApi(object):
    """InterfacesApi unit test stubs"""

    def test_get_interfaces(self, rest_mocker, vns3_client, vns3_api_schema: dict):
        """Test case for get_interfaces

        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "get",
            "/interfaces",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.InterfaceListResponse,
        )(interfaces_api.get_interfaces)

    def test_get_system_interface_details(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for get_system_interface_details

        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "get",
            "/interfaces/system/{interface_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.InterfaceDetail,
        )(interfaces_api.get_system_interface_details)

    def test_get_system_interfaces(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for get_system_interfaces
        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "get",
            "/interfaces/system",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.InterfaceListResponse,
        )(interfaces_api.get_system_interfaces)

    def test_post_create_system_interface(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for post_create_system_interface
        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "post",
            "/interfaces/system",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.InterfaceDetail,
        )(interfaces_api.post_create_system_interface)

    def test_put_update_system_interface(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for put_update_system_interface
        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "put",
            "/interfaces/system/{interface_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.InterfaceDetail,
        )(interfaces_api.put_update_system_interface)

    def test_delete_system_interface(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for delete_system_interface

        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "delete",
            "/interfaces/system/{interface_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.InterfaceDetail,
        )(interfaces_api.delete_system_interface)

    def test_post_interfaces_action(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for post_interfaces_action
        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "post",
            "/interfaces/action",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response_from_schema=True,
        )(interfaces_api.post_interfaces_action)

    def test_get_gre_interfaces(self, rest_mocker, vns3_client, vns3_api_schema: dict):
        """Test case for get_gre_interfaces
        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "get",
            "/interfaces/edge_gre",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.GREEndpointListResponse,
        )(interfaces_api.get_gre_interfaces)

    def test_get_gre_interface_details(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for get_gre_interface_details

        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "get",
            "/interfaces/edge_gre/{interface_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.GREEndpointDetail,
        )(interfaces_api.get_gre_interface_details)

    def test_delete_gre_interface(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for delete_gre_interface

        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "delete",
            "/interfaces/edge_gre/{interface_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.GREEndpointDetail,
        )(interfaces_api.delete_gre_interface)

    def test_post_create_gre_interface(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for post_create_gre_interface
        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "post",
            "/interfaces/edge_gre",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.GREEndpointDetail,
        )(interfaces_api.post_create_gre_interface)

    def test_put_update_gre_interface(
        self, rest_mocker, vns3_client, vns3_api_schema: dict
    ):
        """Test case for put_update_gre_interface
        """
        generate_method_test(
            vns3_client,
            vns3_api_schema,
            "put",
            "/interfaces/edge_gre/{interface_id}",
            rest_mocker,
            mock_request_from_schema=True,
            mock_response=InterfacesApiData.GREEndpointDetail,
        )(interfaces_api.put_update_gre_interface)



# coding: utf-8

"""
    VNS3:ms API

    Cohesive networks VNS3 provides complete control of your network's addresses, routes, rules and edge. Networking does  # noqa: E501

    Contact: solutions@cohesive.net
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

from cohesivenet.api_builder import VersionRouter
from cohesivenet.exceptions import ApiValueError


def put_activate_api_key(
    api_client, username=None, api_key=None, **kwargs
):  # noqa: E501
    """Activate the default API key  # noqa: E501

    Activate the default API key  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_activate_api_key(async_req=True)

    :param VNS3Client api_client: (required)
    :param username str: Username for default API Key
    :param api_key str: Api key to activate
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["username", "api_key"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = []  # noqa: E501

    return api_client.call_api(
        "/auth/activate",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_create_token(api_client, username=None, api_key=None, **kwargs):  # noqa: E501
    """API login method. Accept API-key as password, return API-token to be used for all other calls  # noqa: E501

    API login method. Accept API-key as password, return API-token to be used for all other calls  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_create_token(async_req=True)

    :param VNS3Client api_client: (required)
    :param username str: Username associated with api_key (required)
    :param api_key str: Api key to use for generating token (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["username", "api_key"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = []  # noqa: E501

    return api_client.call_api(
        "/auth",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_expire_token(api_client, **kwargs):  # noqa: E501
    """Expire current token to log out of API session  # noqa: E501

    Expire current token to log out of API session  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_expire_token(async_req=True)

    :param VNS3Client api_client: (required)
    :param username str: Username associated with api_key
    :param api_key str: Api key to use for generating token
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/auth/expire_token",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_invalidate_api_key_tokens(api_client, api_key_id=None, **kwargs):  # noqa: E501
    """Invalidate all tokens for a specific API key  # noqa: E501

    Invalidate all tokens for a specific API key  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_invalidate_api_key_tokens(async_req=True)

    :param VNS3Client api_client: (required)
    :param api_key_id int: api_key to invalidate tokens for
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["api_key_id"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/auth/invalidate_tokens",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_api_keys(api_client, **kwargs):  # noqa: E501
    """Get all API keys  # noqa: E501

    Get all API keys  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_api_keys(async_req=True)

    :param VNS3Client api_client: (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/auth/api_keys",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_create_api_key(api_client, key_name=None, **kwargs):  # noqa: E501
    """Create new API key  # noqa: E501

    Create new API key  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_create_api_key(async_req=True)

    :param VNS3Client api_client: (required)
    :param key_name str: Name or description of API key (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["key_name"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/auth/api_key",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_update_api_key(
    api_client, key_id, key_name=None, enabled=None, **kwargs
):  # noqa: E501
    """Update API key  # noqa: E501

    Update API key  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_update_api_key(async_req=True)

    :param VNS3Client api_client: (required)
    :param key_id int: ID for API key (required)
    :param key_name str: Name or description of API key
    :param enabled bool: Enable/Disable API key
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["key_name", "enabled"]

    collection_formats = {}

    path_params = {"key_id": key_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/auth/api_key/{key_id}",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def delete_api_key(api_client, key_id, **kwargs):  # noqa: E501
    """Delete API key  # noqa: E501

    Delete API key  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.delete_api_key(async_req=True)

    :param VNS3Client api_client: (required)
    :param key_id int: ID for API key (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {"key_id": key_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/auth/api_key/{key_id}",
        "DELETE",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_access_tokens(api_client, **kwargs):  # noqa: E501
    """Get all Access tokens for API  # noqa: E501

    Get all Access tokens for API  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_access_tokens(async_req=True)

    :param VNS3Client api_client: (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/access/tokens",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_create_access_token(api_client, name=None, expires=None, refreshes=None, user_id=None, **kwargs):  # noqa: E501
    """Create an Access token for use with the API # noqa: E501

    Create an Access token for use with the API  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_create_access_token(async_req=True)

    :param VNS3Client api_client: (required)
    :param name str: Name for token (optional)
    :param expires integer: Seconds till expiration (default=3600)
    :param refreshes bool: If true, expiration refreshes on each use
    :param user_id int: If super user, can create tokens for users
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["name", "expires", "refreshes", "user_id"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = []  # noqa: E501

    return api_client.call_api(
        "/access/token",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_access_token(api_client, token_id, **kwargs):  # noqa: E501
    """Get access token details  # noqa: E501

    Get access token details    # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_access_token(token_id, async_req=True)

    :param token_id int: access token id (required)
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = []  # noqa: E501

    collection_formats = {}

    path_params = {"token_id": token_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/access/token/{token_id}",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_update_access_token(api_client, token_id, expired=None, refresh=None, **kwargs):  # noqa: E501
    """Update an Access token # noqa: E501

    Update an Access token   # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_update_access_token(token_id, expired=True, async_req=True)

    :param VNS3Client api_client: (required)
    :param token_id int: access token id (required)
    :param expired bool: True to expire  token
    :param refresh bool: True to refresh expiry time for token
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["expired", "refresh"]

    if not any([expired is not None, refresh is not None]):
        raise ApiValueError("put_update_access_token requires either expired or refresh as parameters")

    collection_formats = {}

    path_params = {"token_id": token_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = []  # noqa: E501

    return api_client.call_api(
        "/access/token/{token_id}",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_regenerate_access_token(api_client, token_id, **kwargs):  # noqa: E501
    """Regenerate an Access token # noqa: E501

    Regenerate an Access token   # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_regenerate_access_token(token_id, async_req=True)

    :param VNS3Client api_client: (required)
    :param token_id int: access token id (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {"token_id": token_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = []  # noqa: E501

    return api_client.call_api(
        "/access/token/{token_id}",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def delete_access_token(api_client, token_id, **kwargs):  # noqa: E501
    """Delete an Access token # noqa: E501

    Delete an Access token   # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.delete_access_token(token_id, async_req=True)

    :param VNS3Client api_client: (required)
    :param token_id int: access token id (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {"token_id": token_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = []  # noqa: E501

    return api_client.call_api(
        "/access/token/{token_id}",
        "DELETE",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_access_urls(api_client, **kwargs):  # noqa: E501
    """Get all Access URLs for UI  # noqa: E501

    Get all Access URLs for UI  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_access_urls(async_req=True)

    :param VNS3Client api_client: (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/access/urls",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def post_create_access_url(api_client, expires=None, description=None, **kwargs):  # noqa: E501
    """Create an Access token for use with the API # noqa: E501

    Create an Access token for use with the API  # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.post_create_access_token(async_req=True)

    :param VNS3Client api_client: (required)
    :param expires integer: Seconds till expiration (default=3600)
    :param description str: Name for token (optional)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["expires", "description"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = []  # noqa: E501

    return api_client.call_api(
        "/access/url",
        "POST",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def get_access_url(api_client, access_url_id, **kwargs):  # noqa: E501
    """Get access URL details  # noqa: E501

    Get access URL details    # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.get_access_url(access_url_id, async_req=True)

    :param access_url_id int: access url id (required)
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = []  # noqa: E501

    collection_formats = {}

    path_params = {"access_url_id": access_url_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = None
    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/access/url/{access_url_id}",
        "GET",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def put_update_access_url(api_client, access_url_id, expired=None, **kwargs):  # noqa: E501
    """Update an Access URL  # noqa: E501

    Update an Access URL   # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.put_update_access_url(access_url_id, expired=True, async_req=True)

    :param VNS3Client api_client: (required)
    :param access_url_id int: access url id (required)
    :param expired bool: True to expire url
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = ["expired"]

    if expired is None:
        raise ApiValueError("put_update_access_url requires expired as parameter")

    collection_formats = {}

    path_params = {"access_url_id": access_url_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = []  # noqa: E501

    return api_client.call_api(
        "/access/url/{access_url_id}",
        "PUT",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def delete_access_url(api_client, access_url_id, **kwargs):  # noqa: E501
    """Delete an Access URL # noqa: E501

    Delete an Access URL   # noqa: E501

    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.delete_access_url(access_url_id, async_req=True)

    :param VNS3Client api_client: (required)
    :param access_url_id int: access url id (required)
    :param async_req bool: execute request asynchronously
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()
    request_params = []

    collection_formats = {}

    path_params = {"access_url_id": access_url_id}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = []  # noqa: E501

    return api_client.call_api(
        "/access/url/{access_url_id}",
        "DELETE",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


def delete_access_url_by_search(
    api_client, access_url_id=None, access_url=None, **kwargs
):  # noqa: E501
    """Find and delete access URL  # noqa: E501

    Delete access URL by ID or URL  # noqa: E501
    This method makes a synchronous HTTP request by default. To make an
    asynchronous HTTP request, please pass async_req=True
    >>> response = await api.delete_access_url_by_search(async_req=True)

    :param VNS3Client api_client: (required)
    :param access_url_id int: ID of access URL
    :param access_url str: URL
    :param async_req bool: execute request asynchronously
    :param _return_http_data_only: response data without head status code
                                    and headers
    :param _preload_content: if False, the urllib3.HTTPResponse object will
                                be returned without reading/decoding response
                                data. Default is True.
    :param _request_timeout: timeout setting for this request. If one
                                number provided, it will be total request
                                timeout. It can also be a pair (tuple) of
                                (connection, read) timeouts.
    :return: APIResponse or awaitable if async
    """

    local_var_params = locals()

    request_params = ["access_url_id", "access_url"]

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = {}
    for param in [p for p in request_params if local_var_params.get(p) is not None]:
        body_params[param] = local_var_params[param]

    # HTTP header `Accept`
    header_params["Accept"] = api_client.select_header_accept(
        ["application/json"]
    )  # noqa: E501

    # HTTP header `Content-Type`
    header_params["Content-Type"] = api_client.select_header_content_type(  # noqa: E501
        ["application/json"]
    )  # noqa: E501

    # Authentication setting
    auth_settings = ["ApiTokenAuth", "basicAuth"]  # noqa: E501

    return api_client.call_api(
        "/access/url",
        "DELETE",
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type="object",  # noqa: E501
        auth_settings=auth_settings,
        async_req=local_var_params.get("async_req"),
        _return_http_data_only=local_var_params.get(
            "_return_http_data_only"
        ),  # noqa: E501
        _preload_content=local_var_params.get("_preload_content", True),
        _request_timeout=local_var_params.get("_request_timeout"),
        collection_formats=collection_formats,
    )


class AccessApiRouter(VersionRouter):
    """Provision API keys or short-lived access URLs for
      providing temporary access to any VNS3 controller in your topology"""

    function_library = {
        "put_activate_api_key": {"2.1.1-2.3.5": put_activate_api_key},
        "post_create_token": {"2.1.1-2.3.5": post_create_token},
        "put_expire_token": {"2.1.1-2.3.5": put_expire_token},
        "put_invalidate_api_key_tokens": {"2.1.1-2.3.5": put_invalidate_api_key_tokens},
        "get_api_keys": {"2.1.1-2.3.5": get_api_keys},
        "post_create_api_key": {"2.1.1-2.3.5": post_create_api_key},
        "put_update_api_key": {"2.1.1-2.3.5": put_update_api_key},
        "delete_api_key": {"2.1.1-2.3.5": delete_api_key},

        "get_access_tokens": {"2.5.1": get_access_tokens},
        "post_create_access_token": {"2.5.1": post_create_access_token},
        "get_access_token": {"2.5.1": get_access_token},
        "put_update_access_token": {"2.5.1": put_update_access_token},
        "post_regenerate_access_token": {"2.5.1": post_regenerate_access_token},
        "delete_access_token": {"2.5.1": delete_access_token},
        "get_access_urls": {"2.5.1": get_access_urls},
        "post_create_access_url": {"2.5.1": post_create_access_url},
        "delete_access_url_by_search": {"2.5.1": delete_access_url_by_search},
        "get_access_url": {"2.5.1": get_access_url},
        "put_update_access_url": {"2.5.1": put_update_access_url},
        "delete_access_url": {"2.5.1": delete_access_url},
    }



from __future__ import absolute_import

import datetime
import json
import os
import re
import tempfile

import cohesivenet.util
from cohesivenet import rest


class Serializer(object):
    def __init__(self, configuration=None):
        self.configuration = None

    PRIMITIVE_TYPES = (float, bool, bytes, int, str)
    NATIVE_TYPES_MAPPING = {
        "int": int,
        "long": int,  # noqa: F821
        "float": float,
        "str": str,
        "bool": bool,
        "date": datetime.date,
        "datetime": datetime.datetime,
        "object": object,
    }

    def sanitize_for_serialization(self, obj):
        """Builds a JSON POST object.

        If obj is None, return None.
        If obj is str, int, long, float, bool, return directly.
        If obj is datetime.datetime, datetime.date
            convert to string in iso8601 format.
        If obj is list, sanitize each element in the list.
        If obj is dict, return the dict.
        If obj is OpenAPI model, return the properties dict.

        :param obj: The data to serialize.
        :return: The serialized form of data.
        """
        if obj is None:
            return None
        elif isinstance(obj, self.PRIMITIVE_TYPES):
            return obj
        elif isinstance(obj, list):
            return [self.sanitize_for_serialization(sub_obj) for sub_obj in obj]
        elif isinstance(obj, tuple):
            return tuple(self.sanitize_for_serialization(sub_obj) for sub_obj in obj)
        elif isinstance(obj, (datetime.datetime, datetime.date)):
            return obj.isoformat()

        if isinstance(obj, dict):
            obj_dict = obj
        else:
            raise RuntimeError("Unkown object type %s" % str(type(obj)))

        return {
            key: self.sanitize_for_serialization(val) for key, val in obj_dict.items()
        }

    def deserialize(self, response, response_type="object"):
        """Deserializes response into an object.

        :param response: RESTResponse object to be deserialized.
        :param response_type: class literal for
            deserialized object, or string of class name.

        :return: deserialized object.
        """
        if response.data is None:
            return None

        if type(response.data) is str:
            error_response_str = re.findall(r"{\"error\".*}", response.data)
            if error_response_str:
                raise rest.ApiException(
                    status=response.status, reason=error_response_str[0]
                )

        # handle file downloading
        # save response body into a tmp file and return the instance
        if "file" in response_type:
            return self.__deserialize_file(response, response_type)

        # fetch data from response object
        try:
            data = json.loads(response.data)
        except ValueError:
            data = response.data

        return self.__deserialize(data, response_type)

    def __deserialize(self, data, klass):
        """Deserializes dict, list, str into an object.

        :param data: dict, list or str.
        :param klass: class literal, or string of class name.

        :return: object.
        """
        if data is None:
            return None

        if type(klass) == str:
            if klass.startswith("list["):
                sub_kls = re.match(r"list\[(.*)\]", klass).group(1)
                return [self.__deserialize(sub_data, sub_kls) for sub_data in data]

            if klass.startswith("dict("):
                sub_kls = re.match(r"dict\(([^,]*), (.*)\)", klass).group(2)
                return {k: self.__deserialize(v, sub_kls) for k, v in data.items()}

            # convert str to class
            if klass in self.NATIVE_TYPES_MAPPING:
                klass = self.NATIVE_TYPES_MAPPING[klass]
            else:
                raise RuntimeError("Unknown native type: %s" % klass)

        if klass in self.PRIMITIVE_TYPES:
            return self.__deserialize_primitive(data, klass)
        elif klass == object:
            return self.__deserialize_object(data)
        elif klass == datetime.date:
            return self.__deserialize_date(data)
        elif klass == datetime.datetime:
            return self.__deserialize_datatime(data)
        else:
            raise RuntimeError("Unknown type %s. Classes not supported." % klass)

    def __deserialize_file(self, response, response_type=None):
        """Deserializes body to file

        Saves response body into a file in a temporary folder,
        using the filename from the `Content-Disposition` header if provided.

        :param response:  RESTResponse.
        :return: file path.
        """
        file_type = ""
        if response_type:
            file_parts = response_type.split(":")
            if len(file_parts) >= 2:
                file_type = file_parts[1]

        fd, path = tempfile.mkstemp(
            dir=(self.configuration.temp_folder_path if self.configuration else None)
        )
        os.close(fd)
        os.remove(path)

        # default to timestamped filename
        filename = cohesivenet.util.random_timestamp_filename(file_type=file_type)
        content_disposition = response.getheader("Content-Disposition")
        if content_disposition:
            content_filename = re.search(
                r'filename=[\'"]?([^\'"\s]+)[\'"]?', content_disposition
            ).group(1)

            if (
                content_filename != "yourfilename"
            ):  # API returns this by default, which is annoying.
                filename = content_filename

        path = os.path.join(os.path.dirname(path), filename)
        response_binary = (
            str.encode(response.data) if type(response.data) is str else response.data
        )
        with open(path, "wb") as f:
            f.write(response_binary)

        return path

    def __deserialize_primitive(self, data, klass):
        """Deserializes string to primitive type.

        :param data: str.
        :param klass: class literal.

        :return: int, long, float, str, bool.
        """
        try:
            return klass(data)
        except UnicodeEncodeError:
            return str(data)
        except TypeError:
            return data

    def __deserialize_object(self, value):
        """Return an original value.

        :return: object.
        """
        return value

    def __deserialize_date(self, string):
        """Deserializes string to date.

        :param string: str.
        :return: date.
        """
        try:
            from dateutil.parser import parse

            return parse(string).date()
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0, reason="Failed to parse `{0}` as date object".format(string)
            )

    def __deserialize_datatime(self, string):
        """Deserializes string to datetime.

        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
        try:
            from dateutil.parser import parse

            return parse(string)
        except ImportError:
            return string
        except ValueError:
            raise rest.ApiException(
                status=0,
                reason=("Failed to parse `{0}` as datetime object".format(string)),
            )
